#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "db-interface.h"

extern FILE* LOGFILE;

#if defined(_WIN32) || defined(_WIN64)
  #include <windows.h>
#endif

#include <postgresql/libpq-fe.h>

Database::Database() {
  result=NULL;
  dbglevel=0;
}

int Database::Connect(const char *host,const char *dbname,const char *uname,const char *passwd) {
  if (dbglevel>=2) {
    fprintf(LOGFILE,"Database::Connect(%s,%s,*)\n",host,uname);
    fflush(LOGFILE);
  }

  char conninfo[8192];
  sprintf(conninfo,"host='%s' dbname='%s' user='%s' password='%s'",host,dbname,uname,passwd);
  db=PQconnectdb(conninfo);
  if (PQstatus(db) == CONNECTION_BAD) {
    if (dbglevel>=1) {
      fprintf(LOGFILE,"Database::Connect: %s\n",Error());
      fflush(LOGFILE);
    }
    PQfinish(db);
    return 0;
  }
  if (dbglevel>=2) {
    fprintf(LOGFILE,"Database::Connect: OK\n");
    fflush(LOGFILE);
  }

  return 1;
}

int Database::Disconnect() {
  if (dbglevel>=2) {
    fprintf(LOGFILE,"%s","Database::Disconnect()");
    fflush(LOGFILE);
  }

  PQfinish(db);
  return 0;
}

void Database::DebugLevel(int l) {
  dbglevel=l;
}

int Database::Errno() {
  if (PQresultStatus(result)>2) {
    if (dbglevel>=1) {
      fprintf(LOGFILE,"Database::Errno: %d\n",PQresultStatus(result));
      fflush(LOGFILE);
    }
  }
  return PQresultStatus(result);
}

char *Database::Error() {
  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::Error: %s\n", PQerrorMessage(db));
    fflush(LOGFILE);
  }
  return PQerrorMessage(db);
}

long Database::NumRows() {
  return numrows;
}

int Database::SQL(const char *sql) {
  if (dbglevel>=3) {
    fprintf(LOGFILE,"Database::SQL(%s)\n",sql);
    fflush(LOGFILE);
  }

  PQclear(result);
  result = NULL;
  rowid  = 0;
  numrows= 0;

  result = PQexec(db,sql);
  int ec=Errno();

  if ((ec==PGRES_EMPTY_QUERY) || (ec==PGRES_COMMAND_OK) || (ec==PGRES_TUPLES_OK)) {
    numrows=PQntuples(result);

    return 1;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::SQL: %s\n",Error());
    fflush(LOGFILE);
  }

  return 0;
}

int Database::NextRow() {
  if (dbglevel>=2) {
    fprintf(LOGFILE,"%s","Database::NextRow");
    fflush(LOGFILE);
  }

  if (result!=NULL)
    if ((0<PQntuples(result)) && (rowid+1<PQntuples(result))) {
      rowid++;
      return 1;
    }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::NextRow: %s\n",Error());
    fflush(LOGFILE);
  }

  return 0;
}

int Database::NotNull(int rc) {
  if (result!=NULL) {
    if ((0<PQntuples(result)) && (rowid<PQntuples(result))) {
      if ((0<PQnfields(result)) && (rc<PQnfields(result))) {
	return !(PQgetisnull(result,rowid,rc));
      }
    }
  }
  return 0;
}

unsigned int Database::GetUInt(int rc) {
  char *vb=GetString(rc);
  unsigned int v;
  if (vb!=NULL) {
    sscanf(vb,"%u",&v);
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetUInt: %s\n",Error());
    fflush(LOGFILE);
  }

  return RAND_MAX;
}

long int Database::GetLong(int rc) {
  char *vb=GetString(rc);
  long v;
  if (vb!=NULL) {
    sscanf(vb,"%ld",&v);
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetLong: %s\n",Error());
    fflush(LOGFILE);
  }
  return RAND_MAX;
}

unsigned long int Database::GetUnsignedLong(int rc) {
  char *vb=GetString(rc);
  if (vb!=NULL) {
    unsigned long v;
    sscanf(vb,"%lu",&v);
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetUnsignedLong: %s\n",Error());
    fflush(LOGFILE);
  }
  return RAND_MAX;
}

int Database::GetInt(int rc) {
  char *vb=GetString(rc);
  int v;
  if (vb!=NULL) {
    sscanf(vb,"%d",&v);
    return v;
  }
  
  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetInt: %s\n",Error());
    fflush(LOGFILE);
  }
  return RAND_MAX;
}

float Database::GetFloat(int rc) {
  char *vb=GetString(rc);
  float v;
  if (vb!=NULL) {
    sscanf(vb,"%f",&v);
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetFloat: %s\n",Error());
    fflush(LOGFILE);
  }
  return (float)1e38;
}

double Database::GetDouble(int rc) {
  char *vb=GetString(rc);
  double v;
  if (vb!=NULL) {
    sscanf(vb,"%lf",&v);
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetDouble: %s\n",Error());
    fflush(LOGFILE);
  }

  return 1e38;
}

char Database::GetChar(int rc) {
  char *vb=GetString(rc);
  char v;
  if (vb!=NULL) {
    v=vb[0];
    return v;
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetChar: %s\n",Error());
    fflush(LOGFILE);
  }

  return (char)255;
}

char *Database::GetString(int rc) {
  if (result!=NULL) {
    if ((0<PQntuples(result)) && (rowid<PQntuples(result))) {
      if ((0<PQnfields(result)) && (rc<PQnfields(result))) {
	return PQgetvalue(result,rowid,rc);
      }
    }
  }

  if (dbglevel>=1) {
    fprintf(LOGFILE,"Database::GetString: %s\n",Error());
    fflush(LOGFILE);
  }

  return NULL;
}

Database::~Database() {
  if (dbglevel>=2) {
    fprintf(LOGFILE,"Database::~Database\n");
    fflush(LOGFILE);
  }
}
